#==============================================================
# sphere.py  - Auteur Claude Turrier - janvier 2021
# Ce programme génère un fichier sphere.obj (sphère 3d texturée)
# r: rayon de la sphère
# pa : nombre de parallèles (nombre impaire supérieur ou égal à 3)
# me: nombre de méridiens (nombre pair supérieur ou égal à 4)
# Utilisation:
# 1) fixer les valeurs de r, pa et me dans le prgoramme (lignes 21 à 23)
# 2) Lancer le programme
# 3) Placer dans le même répertoire : le fichier sphere.obj,
#    (généré automatiquement par le programme sphere.py 
#    et les fichiers sphere.mtl et texture.png ci-joints
# 4) Ouvrir sphere.obj avec meshlab 
#=================================================================
import math as m
import sys
import os
#---------------------------------------------------------------
# Valeurs à fixer
#---------------------------------------------------------------
r=1.0
pa=99 # nombre de parallèles (nombre impair supérieur ou égal à 3)
me=100 # nombre de méridiens (nombre pair supérieur ou égal à 4)
#---------------------------------------------------------------
# Variables du programme
#---------------------------------------------------------------
pi=m.pi; v="v";vt="vt";f="f"
ns=me*pa #nombre de sommets
nc=pa*me+pa #nombre de coordonnées de texture
nf=(pa-1)*me #nombre de faces quadrangulaires (cotés)
nft=(me-2)*2 #nombre de faces triangulaires (dessus et dessous)
phi= pi/(2*pa)
theta=2*pi/me
#---------------------------------------------------------------
# Redirection de la sortie print vers le fichier sphere.obj
#---------------------------------------------------------------
if os.path.exists('sphere.obj'):
    os.remove("sphere.obj")
temp = sys.stdout                 
sys.stdout = open('sphere.obj', 'a') 
#---------------------------------------------------------------
# Création du fichier sphere.obj
#---------------------------------------------------------------
print ("######################################################")
print ("# SPHERE 3D")
print ("# OBJ File sphere.obj généré par sphere.py")
print ("# Auteur Claude Turrier - Janvier 2021")
print ("# Parallèles: %3d"  %(pa))
print ("# Méridiens: %3d"  %(me))
print ("# Sommets: %3d"  %(ns))
print ("# Faces quadrangulaires: %4d (cotés)"  %(nf))
print ("# Faces triangulaires: %4d (dessus et dessous)"  %(nft))
print ("# Coordonnées de texture: %4d"  %(nc))
print ("######################################################")
print (" ")
print("mtllib ./sphere.mtl")
print("usemtl material_0")
print()      
#----------------------------------------------------------
# ns=me*pa=sommets v (ns=18 pour une sphere pa,pe=3,6)
#----------------------------------------------------------
for i in range (1,2*pa,2) :#formule générale
  for j in range (0,me) :
   x=r*m.sin(i*phi)*m.cos(j*theta)
   y=r*m.sin(i*phi)*m.sin(j*theta)
   z= r*m.cos(i*phi)
   print ("%1s %4f %4f %4f" %(v, x, y, z))
#----------------------------------------------------------
# nc=pa*me+pa coordonnées de textures (nc=21 pour une sphere pa,pe=3,6)
#-----------------------------------------------------------------
#vt numéro 1 à numéro pa*me
pas_horiz=1/me
pas_vert=1/(pa-1)
for i in range (0,pa) :
  for j in range (0,me) :
   u=0.0+pas_horiz*j
   v=1.0-i*pas_vert
   print ("%1s %4f %4f" %(vt, u, v))
#vt numéro pa*me+1 à numéro pa*me+pa
for i in range (0,pa) :
   u=1.0
   v=1.0-i*pas_vert
   print ("%1s %4f %4f" %(vt, u, v))
#----------------------------------------------------------
# nf=(pa-1)*me=12 faces (nf=18 pour une sphere pa,pe=3,6)
#----------------------------------------------------------
#faces numéro 1 à (me-1)*(pa-1)
for i in range (1,pa) :#formule générale
  for j in range (1,me) :
   v1=i*me+j; vt1=v1;
   v2=v1+1; vt2=v2
   v3=v1-me+1; vt3=v3
   v4=v3-1; vt4=v4
   v1=str(v1);vt1=str(vt1);
   v2=str(v2);vt2=str(vt2);
   v3=str(v3);vt3=str(vt3);
   v4=str(v4);vt4=str(vt4);
   chaine= f + " " +v1+"/"+vt1 + " "+  v2+"/"+vt2 + " " + v3+"/"+vt3  + " " + v4+"/"+vt4 
   print ("%1s" %chaine)  
#faces numéro (me-1)*(pa-1)+1 à me*(pa-1)
for i in range (1,pa) :
   v1=me+i*me; vt1=v1;
   v2=i*me+1; vt2=ns+i+1;
   v3=1+(i-1)*me; vt3=ns+i
   v4=i*me; vt4=v4
   v1=str(v1);vt1=str(vt1);
   v2=str(v2);vt2=str(vt2);
   v3=str(v3);vt3=str(vt3);
   v4=str(v4);vt4=str(vt4);
   chaine= f + " " +v1+"/"+vt1 + " "+  v2+"/"+vt2 + " " + v3+"/"+vt3  + " " + v4+"/"+vt4 
   print ("%1s" %chaine)  
#faces du dessus (triangulaires)
for i in range (1, me-1) :   
  v1=1;vt1=8                     
  v2=i+1;vt2=2   
  v3=i+2 ;vt3=1 
  v1=str(v1);vt1=str(8);
  v2=str(v2);vt2=str(2);
  v3=str(v3);vt3=str(1);
  chaine= f + " " +v1+"/"+vt1 + " "+  v2+"/"+vt2 + " " + v3+"/"+vt3  
  print ("%1s" %chaine)
#faces du dessous (triangulaires)
for i in range (1, me-1) :   
  v1=ns;vt1=8                     
  v2=ns-i;vt2=2   
  v3=ns-i-1 ;vt3=1 
  v1=str(v1);vt1=str(8);
  v2=str(v2);vt2=str(2);
  v3=str(v3);vt3=str(1);
  chaine= f + " " +v1+"/"+vt1 + " "+  v2+"/"+vt2 + " " + v3+"/"+vt3  
  print ("%1s" %chaine)   
#---------------------------------------------------------------
# Redirection de la sortie print vers l'écran
#--------------------------------------------------------------- 
sys.stdout.close()                
sys.stdout = temp                 
